@@ -25,7 +25,7 @@ Backtrace::Backtrace (const Backtrace& other) noexcept : buffer(other.buffer) {}
 Backtrace::Backtrace () noexcept {
     buffer.resize(max_depth);
     auto depth = ::backtrace(buffer.data(), max_depth);
-    if (depth == -1) {
+    if (depth <= 0) {
         buffer.clear();
         return;
     } 
